#include "mcc.h"

/* main */
int
main(int args, char *argv[])
{
   struct hostent     *server_ent;
   struct sockaddr_in  servname;
   int    port;             // puerto a usar
   long int test;           // var temp para revisar validez del argumento <port>
   
   char   bufout[BUFSIZE];  // buffer de salida
   char   bufin[BUFSIZE];   // buffer de entrada
   size_t charcount;        // cuenta caracteres leidos o escritos por read, write, send y recv
   
   int f_ret;               // recuerda valor de retorno de algunas funciones
   
   fd_set atento;           // usado por select()
   fd_set _atento;          // backup de la variable anterior
   
   char *pmyname;           // puntero temporal para determinar nombre de usuario
   char myname[NAMESIZE];   // nombre de usuario
   
   /* revisa cantidad correcta de argumentos */
   if (args != 3)
   {
      printf("Uso: %s server port\n", argv[0]);
      exit(1);
   }
   
   /* revisa formato y valor aceptable de argumento <port> */
   test = strtol(argv[2], NULL, 0);
   if (errno == EINVAL)
   {
      printf("Argumento invalido: port\n");
      exit(1);
   }
   if (errno == ERANGE || test > 0xFFFF || test < 0)
   {
      printf("numero invalido de puerto: %ld\n", test);
      exit(1);
   }
   port = (int)test;
   
   signal(SIGINT, sig_handler);
   
   /* obteniendo nombre de usuario */
   pmyname = getpwuid(geteuid())->pw_name;
   if (pmyname == NULL)
   {
      printf("no se pudo determinar el nombre de usuario\n");
      exit(1);
   }
   strncpy(myname, pmyname, NAMESIZE);
   myname[NAMESIZE-1] = 0; // para asegurar fin de string con '\0'
   
   server_ent = gethostbyname(argv[1]);
   if (server_ent == NULL )
   {
      perror("gethostbyname");
      exit(1);
   }
   
   servname.sin_family = AF_INET;
   servname.sin_port   = htons(port);
   memcpy(&servname.sin_addr, server_ent->h_addr_list[0], server_ent->h_length);
   
   out_sock = socket(AF_INET, SOCK_STREAM, 0);
   
   atexit(close_ritual);
   
   printf("Conectando con %s\n", inet_ntoa(servname.sin_addr));
   f_ret = connect(out_sock, (struct sockaddr *)&servname, sizeof(struct sockaddr_in));
   if (f_ret)
   {
      perror("connect");
      exit(1);
   }
   
   printf("solicitando servicio de chat\n");
   
   /* envia nombre de usuario al servidor */
   charcount = send(out_sock, myname, strlen(myname), 0);
   if (charcount < 0)
   {
      perror("myname, send");
      exit(1);
   }
   printf("para salir use (Control+D)\n\n");
   
   /* preparativos para select() */
   FD_ZERO(&_atento);
   FD_SET(0, &_atento);        // quiero escuchar a stdin (teclado)
   FD_SET(out_sock, &_atento); // quiero escuchar al socket
   
   do
   {
      atento = _atento;
      f_ret = select(out_sock+1, &atento, NULL, NULL, 0);
      
      if (f_ret == 0)
      {
         printf("select devolvio 0\n");
         exit(1);
      }
      
      // revisa teclado 1° pq es el - activo -> menor prob bloq al otro (al socket)
      if (FD_ISSET(0, &atento))
      {
         /* lee teclado y envia por socket */
         charcount = read(0, bufout, BUFSIZE);
         switch (charcount)
         {
            case 0:
               printf("leido EOF en teclado, fin comunicacion\n");
               exit(1);
            
            case -1:
               printf("error de lectura de teclado, fin de la comunicacion\n");
               exit(1);
         }
         charcount = send(out_sock, bufout, charcount, 0);
         if (charcount < 0)
         {
            perror("send");
            exit(1);
         }
      }
      else
      if (FD_ISSET(out_sock, &atento))
      {
         /* lee socket y escribe en pantalla */
         charcount = recv(out_sock, bufin, BUFSIZE, 0);
         switch (charcount)
         {
            case 0:
               printf("leido EOF en socket, el servidor cerro la comunicacion\n");
               exit(1);
            
            case -1:
               printf("error de lectura en socket, fin de la comunicacion\n");
               exit(1);
         }
         charcount = write(1, bufin, charcount);
         if (charcount < 0)
         {
            perror("write bufin");
            exit(1);
         }
         charcount = write(1, "\n", 1);
         if (charcount < 0)
         {
            perror("write nl");
            exit(1);
         }
      }
      else
      {
         printf("ups: actividad en descriptor inesperado\n");
         exit(1);
      }
      
   }while(1);
   
   //close(out_sock);
   
   return 0;
}

void
sig_handler(int sign)
{
   exit(1);
}

void
close_ritual(void)
{
   close(out_sock);
   printf("Todo bien cerrado\n");
   return;
}
